// Copyright 2000 by David Rhodes and Robert Dick.
// All rights reserved.

//Test
#include <cmath>
#include <iostream.h>
#include <iomanip.h>
#include <functional>

#include "RVector.h"
#include "ArgPack.h"
#include "RStd.h"

/*###########################################################################*/
RString ArgPack::tg_label_ = "TASK_GRAPH";
int ArgPack::tg_cnt_ = 3;
double ArgPack::vertex_time_ = 100;

double ArgPack::vertex_cnt_av_ = 20;
double ArgPack::vertex_cnt_mul_ = 5;

int ArgPack::vertex_in_deg_ = 5;
int ArgPack::vertex_out_deg_ = 5;

bool ArgPack::task_unique_ = false;

int ArgPack::task_type_cnt_ =
  static_cast<int>((ArgPack::tg_cnt_ * ArgPack::vertex_cnt_av_) / 3);

bool ArgPack::trans_unique_ = false;
int ArgPack::trans_type_cnt_ =
  static_cast<int>((ArgPack::tg_cnt_ * ArgPack::vertex_cnt_av_ *
  (ArgPack::vertex_in_deg_ + ArgPack::vertex_out_deg_)) / 12);

RVector<RString> ArgPack::task_attrib_name_;
RVector<double> ArgPack::task_attrib_av_;
RVector<double> ArgPack::task_attrib_mul_;
RVector<double>	ArgPack::task_attrib_round_;

double ArgPack::p_laxity_ = 1;
bool ArgPack::p_greater_deadline_ = false;
double ArgPack::prob_hard_deadline_ = 1.0;
double ArgPack::soft_deadline_mul_ = 1.0;

RVector<int> ArgPack::period_mul_(3);

double ArgPack::entries_per_type_av_ = 1.0;
double ArgPack::entries_per_type_mul_ = 0.0;

bool ArgPack::urv_merge_ = true;
RString ArgPack::table_label_ = "PE";
int ArgPack::table_cnt_ = 5;

double ArgPack::table_type_ratio_ = 0.5;

RVector<RString> ArgPack::table_name_(1);
RVector<double> ArgPack::table_av_(1);
RVector<double> ArgPack::table_mul_(1);
RVector<double> ArgPack::table_jitter_(1);
RVector<double> ArgPack::table_round_(1);

RVector<RString> ArgPack::type_name_(1);
RVector<double> ArgPack::type_av_(1);
RVector<double> ArgPack::type_mul_(1);
RVector<double> ArgPack::type_jitter_(1);
RVector<double> ArgPack::type_round_(1);

int ArgPack::seed_ = 0;
int ArgPack::misc_type_cnt_ = 0;

RString ArgPack::data_file_name_ = "tgff.tgff";
RString ArgPack::eps_file_name_ = "tgff.eps";
RString ArgPack::opt_file_name_ = "tgff.tgffopt";

/*===========================================================================*/
void ArgPack::init(int argc, char *const argv[]) {
	period_mul_[0] = 1;
	period_mul_[1] = 2;
	period_mul_[2] = 3;

	table_name_[0] = "price";
	table_av_[0] = 10.0;
	table_mul_[0] = 5.0;
	table_jitter_[0] = 0.5;
	table_round_[0] = 1.0;

	type_name_[0] = "exec_time";
	type_av_[0] = 100.0;
	type_mul_[0] = 40.0;
	type_jitter_[0] = 0.5;
	type_round_[0] = 1.0;

	if (argc != 2) {
		help(cout);
		rexit();
	}

	data_file_name_ = RString(argv[1]) + ".tgff";
	eps_file_name_ = RString(argv[1]) + ".eps";
	opt_file_name_ = RString(argv[1]) + ".tgffopt";
}

/*===========================================================================*/
void ArgPack::help(ostream &os) {
os <<
"tgff [filename]\n"
"    reads commands from the [filename].tgffopt file\n"
"    writes PostScript to the [filename].eps file\n"
"    writes data to the [filename].tgff file\n"
"\n"
"A \\ can be used to enter multi-line commands.\n"
"A # at the start of a line comments out the line.\n"
"\n"
"TGFF understands the following data types:\n"
"------------------------------------------\n"
"  <string>: a string of characters\n"
"  <int>: an integer number\n"
"  <flt>: a floating point number\n"
"  <bool>: a boolean value which can be ``true'' or ``false''\n"
"  <list(T)>: a comma separated series of zero or more values of type T\n"
"    parentheses are a meta-character, and are not entered in the input file\n"
"\n"
"TGFF understands the following variable setting commands:\n"
"---------------------------------------------------------\n"
"  seed <int>: seeds the pseudo-random number generator\n"
"  tg_label <string>: label used for task graphs\n"
"  tg_cnt <int>: number of task graphs\n"
"  task_trans_time <flt>: average time per task including communication\n"
"        (used in setting deadlines)\n"
"  task_cnt <int> <int>: number of tasks per graph (average, multiplier)\n"
"  task_degree <int> <int>: maximum number of transmits (in, out) per task\n"
"  task_unique <bool>: tasks types are forced to be unique (true by default) \n"
"  task_type_cnt <int>: number of task types \n"
"  trans_type_cnt <int>: number of transmit types\n"
"  task_attrib <list(<string> <flt> <flt> <flt>)>:\n"
"    name, average, multiplier, round to (default 1.0, 0.0 means no rounding)\n"
"  period_laxity <flt>: laxity of periods, relative to deadlines (default 1)\n"
"  period_g_deadline <bool>: periods forced > deadlines (default true)\n"
"  prob_hard_deadline <flt>: probability that a deadline will be hard (vs. soft)\n"
"  soft_deadline_mul <flt>: multiplier applied to soft deadlines (default 1)\n"
"  period_mul <list(<int>)>: multipliers for periods in multirate systems\n"
"\n"
"  entries_per_type <flt> <flt>:\n"
"  number of attribute entries per task type for tables (average, multiplier)\n"
"  table_label <string>: label used for tables\n"
"  table_cnt <int>: number of tables\n"
"  urv_merge <bool>: preserve uniform distribution when merging variables\n"
"                    (default true)\n"
"  type_table_ratio <flt>: ratio of statistical contributions\n"
"                          of type to table attributes\n"
"\n"
"  table_attrib <list(<string> <flt> <flt> <flt> <flt>)>:\n"
"    name, average, multiplier, jitter (default 0.5),\n"
"    round to (default 1.0, 0.0 means no rounding)\n"
"\n"
"  type_attrib <list(<string> <flt> <flt> <flt> <flt>)>:\n"
"    name, average, multiplier, jitter (default 0.5),\n"
"    round to (default 1.0, 0.0 means no rounding)\n"
"\n"
"TGFF understands the following general commands:\n"
"------------------------------------------------\n"
"  eps_write: make a PostScript plot of the task graphs   [to .eps  file]\n"
"  tg_write: write the task graphs                        [to .tgff file]\n"
"  pe_write: write PE information                         [to .tgff file]\n"
"  trans_write: write transmission event information      [to .tgff file]\n"
"  misc_write: write independant processor information    [to .tgff file]\n"
"  note_write <list(<string>)>: write the string(s)       [to .tgff file]\n"
"\n"
"  pack_schedule <int> <int> <flt> <flt> <flt> <flt> ...  [to .tgff file]\n"
"            ... <int> <int>  <int> <int> [<flt>]         [&  .eps file]\n"
"           [... on one line]\n"
"  ** this is a `self-contained' command, it generates PEs, COMs, TG, etc.\n"
"     other writes except note_write should not be used. Only non-periodic\n"
"     instances are generated, 'task_degree' and 'seed' are only cmds that \n"
"     affect its operation\n"
"  ** args in-order are:\n"
"     num_task_graphs avg_task_graphs_per_pe\n"
"     avg_task_time mul_task_time task_slack task_round num_pe_types num_pe_soln\n"
"     num_com_types num_com_soln and optionally arc_fill_factor\n"
"\n";
}
