// Copyright 2000 by David Rhodes and Robert Dick.
// All rights reserved.

#include <cmath>
#include <iostream.h>
#include <algorithm>
#include <set>

#include "RMath.h"
#include "TGraph.h"
#include "ArgPack.h"
#include "RGen.h"

/*###########################################################################*/
TGraph::TGraph(int tg_offset) :
	dag_(ArgPack::tg_cnt_),
	tg_offset_(tg_offset)
{

	if (! dag_.size()) return;

	int first_task = 0;
	MAP(x, dag_.size()) {
		double low_bound = ArgPack::vertex_cnt_mul_ ?
		  RGen::gen.flat_range_d(ArgPack::vertex_cnt_av_ -
		  ArgPack::vertex_cnt_mul_,
		  ArgPack::vertex_cnt_av_ + ArgPack::vertex_cnt_mul_) :
		  ArgPack::vertex_cnt_av_;

		low_bound *= ArgPack::period_mul_[RGen::gen.flat_range_l(0,
		  ArgPack::period_mul_.size())];

		dag_[x].init(low_bound, ArgPack::vertex_in_deg_,
		  ArgPack::vertex_out_deg_, x, first_task);

		first_task += dag_[x].NumNodes();
	}

// Make an array containing all of the maximum deadlines.
	RVector<double> deadline(dag_.size());
	MAP(x, dag_.size()) {
		deadline[x] = dag_[x].max_h() * ArgPack::vertex_time_;
#if 0
// This was used before 14 June 1999.
		deadline[x] = dag_[x].max_h() * ArgPack::vertex_time_ *
		  ArgPack::tg_cnt_;
#endif

	}

// Generate a priority map to order deadline values.
	RVector<int> pri(deadline.size());
	pri_map(deadline.begin(), deadline.end(), pri.begin());

	double max_deadline = deadline[pri.back()];

/* Generate an array of multiples by randomly picking out
of the period_mul_ list. */
	RVector<unsigned> per(dag_.size());
	MAP(x, per.size()) {
		int size = ArgPack::period_mul_.size();
		int index = RGen::gen.flat_range_l(0, size);
		per[x] = ArgPack::period_mul_[index];
	}

// Sort the period multipliers.
	sort(per.begin(), per.end());
	int max_multiple = per.back();

// Figure out the periods.

	double deadline_grain = max_deadline / max_multiple * ArgPack::p_laxity_;

//cout << "max_deadline: " << max_deadline << ", max_multiple: " <<
//  max_multiple << ", p_laxity_: " << ArgPack::p_laxity_ << "\n";
//cout << "deadline_grain: " << deadline_grain << "\n";

// Round to the nearest vertex_time_.
	deadline_grain = interval_round(deadline_grain, ArgPack::vertex_time_);

//cout << "rounded deadline_grain: " << deadline_grain << "\n";

/* Associate the increasing multipliers with graphs in order of
increasing deadlines. */
	MAP(x, per.size())
		dag_[pri[x]].set_period(per[x] * deadline_grain);

// Assure that periods >= deadlines if requested.
	if (ArgPack::p_greater_deadline_) {
		double new_grain = deadline_grain;

		MAP(x, dag_.size()) {
			double old_period = dag_[x].period();
			dag_[x].push_period();

			double period_dif = dag_[x].period() - old_period;
			if (period_dif > 0.0) {
				new_grain =
				  max(new_grain, deadline_grain + period_dif / per[x]);
			}
		}

		deadline_grain = new_grain;
		MAP(x, dag_.size()) {
			dag_[pri[x]].set_period(per[x] * deadline_grain);
		}
	}

// Compute the hyperperiod
	h_period_ = deadline_grain * lcm(per);

//cout << "h_period_: " << h_period_ << "\n";

// Count the total number of task types.
// Correct the ArgPack if necessary.

	if (ArgPack::task_unique_) {
		ArgPack::task_type_cnt_ = 0;

		MAP(x, dag_.size())
			ArgPack::task_type_cnt_ += dag_[x].NumNodes();
	}
}

/*===========================================================================*/
void TGraph::print_to(ostream &os) const {
	MAP(x, dag_.size()) {
		os << "@" << ArgPack::tg_label_ << " " <<
		  (x + tg_offset_) << " {\n" << dag_[x];
	}

	os << "\n\n";
}
