// Copyright 2000 by David Rhodes and Robert Dick.
// All rights reserved.

#ifndef TT_DAG_H_
#define TT_DAG_H_

/*###########################################################################*/
#include "RString.h"
#include "dagman.h"
template <class T> class RVector;

class ostream;
class TTnode;
class TTarc;
class TTdag;

/*###########################################################################*/
class TTnode : public NodeObj {
public:
	TTnode(void);
	TTnode(int i);
	TTnode(int i, int t, unsigned ri, unsigned ru);
	int			type;		// if == -1, then << generates
	unsigned	ri_;		// indx of resource it runs in (for pack_sched)
	unsigned	ru_;		// indx of resourceuse rep by it (for pack_sched)
	int			name;
	double		deadline;
	bool		hard;
	int			h, w;
	RVector<double> attrib_;
};
typedef Listman<TTnode> TTnodeList;
typedef SubListman TTnodeSubList;

/*===========================================================================*/
class TTarc : public ArcObj {
public:
	TTarc(void)		{name = -1; type = -1;};
	TTarc(int n)	{name = n; type = -1;};
	TTarc(int n, unsigned t) {name = n; type = t;};
	int	type;		// if == -1, then << generates
	int name;
};
typedef Listman<TTarc>  TTarcList;

/*===========================================================================*/
class TTdag : public Dagman<TTnode,TTarc> {
public:
// only constructor
	TTdag();

// augment with (built in) pattern
	void init(int low_bound, int in_deg, int out_deg, int number,
	  int first_task);

	void init(int number, int in_deg, int out_deg, double period);

	void print_to(ostream &os) const;
	friend ostream &operator<<(ostream &os, /**/ const TTdag &dag)
	  {dag.print_to(os); return os;}
	friend void	i_PS_dag (ostream&, RVector<TTdag> &dag, int);

	void push_period();
	double period() const {return period_;}
	int in_deg() const {return in_deg_;}
	int out_deg() const {return out_deg_;}
	void set_period(double p) {period_ = p;}
	RString gen_name(const char *header, int name) const;

	int max_w() const {return maxw;}
	int max_h() const {return maxh;}

	void compute_dims();

private:
	void augment();
	void deadlines();
	int parents_h(TTnode *n);

// dag max width and height
		int maxw;
		int maxh;

		int low_bound_;
		int in_deg_;
		int out_deg_;
		double period_;
		int number_;
		int first_task_;
};

/*###########################################################################*/
#endif


